@@ -1,39 +1,48 @@
+import gzip
+import io
 import mimetypes
 import warnings
 from datetime import timedelta
 from tempfile import SpooledTemporaryFile
 
-from django.core.exceptions import ImproperlyConfigured, SuspiciousOperation
+from django.core.exceptions import ImproperlyConfigured
+from django.core.exceptions import SuspiciousOperation
 from django.core.files.base import File
 from django.utils import timezone
 from django.utils.deconstruct import deconstructible
-from django.utils.encoding import force_bytes
 
 from storages.base import BaseStorage
-from storages.utils import (
-    check_location, clean_name, get_available_overwrite_name, safe_join,
-    setting,
-)
+from storages.compress import CompressedFileMixin
+from storages.utils import check_location
+from storages.utils import clean_name
+from storages.utils import get_available_overwrite_name
+from storages.utils import safe_join
+from storages.utils import setting
+from storages.utils import to_bytes
 
 try:
     from google.cloud.exceptions import NotFound
-    from google.cloud.storage import Blob, Client
+    from google.cloud.storage import Blob
+    from google.cloud.storage import Client
     from google.cloud.storage.blob import _quote
+    from google.cloud.storage.retry import DEFAULT_RETRY
 except ImportError:
     raise ImproperlyConfigured("Could not load Google Cloud Storage bindings.\n"
                                "See https://github.com/GoogleCloudPlatform/gcloud-python")
 
 
+CONTENT_ENCODING = 'content_encoding'
 CONTENT_TYPE = 'content_type'
 
 
-class GoogleCloudFile(File):
+class GoogleCloudFile(CompressedFileMixin, File):
     def __init__(self, name, mode, storage):
         self.name = name
         self.mime_type = mimetypes.guess_type(name)[0]
         self._mode = mode
         self._storage = storage
-        self.blob = storage.bucket.get_blob(name)
+        self.blob = storage.bucket.get_blob(
+            name, chunk_size=storage.blob_chunk_size)
         if not self.blob and 'w' in mode:
             self.blob = Blob(
                 self.name, storage.bucket,
@@ -56,6 +65,8 @@ class GoogleCloudFile(File):
                 self._is_dirty = False
                 self.blob.download_to_file(self._file)
                 self._file.seek(0)
+            if self._storage.gzip and self.blob.content_encoding == 'gzip':
+                self._file = self._decompress_file(mode=self._mode, file=self._file)
         return self._file
 
     def _set_file(self, value):
@@ -76,7 +87,7 @@ class GoogleCloudFile(File):
         if 'w' not in self._mode:
             raise AttributeError("File was not opened in write mode.")
         self._is_dirty = True
-        return super().write(force_bytes(content))
+        return super().write(to_bytes(content))
 
     def close(self):
         if self._file is not None:
@@ -84,6 +95,7 @@ class GoogleCloudFile(File):
                 blob_params = self._storage.get_object_parameters(self.name)
                 self.blob.upload_from_file(
                     self.file, rewind=True, content_type=self.mime_type,
+                    retry=DEFAULT_RETRY,
                     predefined_acl=blob_params.get('acl', self._storage.default_acl))
             self._file.close()
             self._file = None
@@ -109,6 +121,14 @@ class GoogleCloudStorage(BaseStorage):
             "default_acl": setting('GS_DEFAULT_ACL'),
             "querystring_auth": setting('GS_QUERYSTRING_AUTH', True),
             "expiration": setting('GS_EXPIRATION', timedelta(seconds=86400)),
+            "gzip": setting('GS_IS_GZIPPED', False),
+            "gzip_content_types": setting('GZIP_CONTENT_TYPES', (
+                'text/css',
+                'text/javascript',
+                'application/javascript',
+                'application/x-javascript',
+                'image/svg+xml',
+            )),
             "file_overwrite": setting('GS_FILE_OVERWRITE', True),
             "cache_control": setting('GS_CACHE_CONTROL'),
             "object_parameters": setting('GS_OBJECT_PARAMETERS', {}),
@@ -154,6 +174,14 @@ class GoogleCloudStorage(BaseStorage):
             raise FileNotFoundError('File does not exist: %s' % name)
         return file_object
 
+    def _compress_content(self, content):
+        content.seek(0)
+        zbuf = io.BytesIO()
+        with gzip.GzipFile(mode='wb', fileobj=zbuf, mtime=0.0) as zfile:
+            zfile.write(to_bytes(content.read()))
+        zbuf.seek(0)
+        return zbuf
+
     def _save(self, name, content):
         cleaned_name = clean_name(name)
         name = self._normalize_name(cleaned_name)
@@ -164,14 +192,24 @@ class GoogleCloudStorage(BaseStorage):
         upload_params = {}
         blob_params = self.get_object_parameters(name)
         upload_params['predefined_acl'] = blob_params.pop('acl', self.default_acl)
+        upload_params[CONTENT_TYPE] = blob_params.pop(CONTENT_TYPE, file_object.mime_type)
 
-        if CONTENT_TYPE not in blob_params:
-            upload_params[CONTENT_TYPE] = file_object.mime_type
+        if (self.gzip and
+                upload_params[CONTENT_TYPE] in self.gzip_content_types and
+                CONTENT_ENCODING not in blob_params):
+            content = self._compress_content(content)
+            blob_params[CONTENT_ENCODING] = 'gzip'
 
         for prop, val in blob_params.items():
             setattr(file_object.blob, prop, val)
 
-        file_object.blob.upload_from_file(content, rewind=True, size=content.size, **upload_params)
+        file_object.blob.upload_from_file(
+            content,
+            rewind=True,
+            retry=DEFAULT_RETRY,
+            size=getattr(content, 'size', None),
+            **upload_params
+        )
         return cleaned_name
 
     def get_object_parameters(self, name):
@@ -194,7 +232,7 @@ class GoogleCloudStorage(BaseStorage):
     def delete(self, name):
         name = self._normalize_name(clean_name(name))
         try:
-            self.bucket.delete_blob(name)
+            self.bucket.delete_blob(name, retry=DEFAULT_RETRY)
         except NotFound:
             pass
 
@@ -267,11 +305,11 @@ class GoogleCloudStorage(BaseStorage):
         created = blob.time_created
         return created if setting('USE_TZ') else timezone.make_naive(created)
 
-    def url(self, name):
+    def url(self, name, parameters=None):
         """
-        Return public url or a signed url for the Blob.
-        This DOES NOT check for existance of Blob - that makes codes too slow
-        for many use cases.
+        Return public URL or a signed URL for the Blob.
+
+        The existnce of blobs are not verified for public URLs, it makes the code too slow.
         """
         name = self._normalize_name(clean_name(name))
         blob = self.bucket.blob(name)
@@ -286,16 +324,19 @@ class GoogleCloudStorage(BaseStorage):
                 storage_base_url=self.custom_endpoint,
                 quoted_name=_quote(name, safe=b"/~"),
             )
-        elif not self.custom_endpoint:
-            return blob.generate_signed_url(
-                expiration=self.expiration, version="v4"
-            )
         else:
-            return blob.generate_signed_url(
-                bucket_bound_hostname=self.custom_endpoint,
-                expiration=self.expiration,
-                version="v4",
-            )
+            default_params = {
+                "bucket_bound_hostname": self.custom_endpoint,
+                "expiration": self.expiration,
+                "version": "v4",
+            }
+            params = parameters or {}
+
+            for key, value in default_params.items():
+                if value and key not in params:
+                    params[key] = value
+
+            return blob.generate_signed_url(**params)
 
     def get_available_name(self, name, max_length=None):
         name = clean_name(name)
