@@ -3,26 +3,37 @@ import os
 import posixpath
 import tempfile
 import threading
-from datetime import datetime, timedelta
-from gzip import GzipFile
+from datetime import datetime
+from datetime import timedelta
 from tempfile import SpooledTemporaryFile
-from urllib.parse import parse_qsl, urlencode, urlsplit
+from urllib.parse import parse_qsl
+from urllib.parse import urlencode
+from urllib.parse import urlsplit
 
 from django.contrib.staticfiles.storage import ManifestFilesMixin
-from django.core.exceptions import ImproperlyConfigured, SuspiciousOperation
+from django.core.exceptions import ImproperlyConfigured
+from django.core.exceptions import SuspiciousOperation
 from django.core.files.base import File
 from django.utils.deconstruct import deconstructible
 from django.utils.encoding import filepath_to_uri
-from django.utils.timezone import is_naive, make_naive
+from django.utils.timezone import is_naive
+from django.utils.timezone import make_naive
 
 from storages.base import BaseStorage
-from storages.utils import (
-    GzipCompressionWrapper, check_location, get_available_overwrite_name,
-    lookup_env, safe_join, setting, to_bytes,
-)
+from storages.compress import CompressedFileMixin
+from storages.compress import CompressStorageMixin
+from storages.utils import check_location
+from storages.utils import clean_name
+from storages.utils import get_available_overwrite_name
+from storages.utils import is_seekable
+from storages.utils import lookup_env
+from storages.utils import safe_join
+from storages.utils import setting
+from storages.utils import to_bytes
 
 try:
     import boto3.session
+    from boto3.s3.transfer import TransferConfig
     from botocore.client import Config
     from botocore.exceptions import ClientError
     from botocore.signers import CloudFrontSigner
@@ -36,9 +47,7 @@ def _use_cryptography_signer():
     from cryptography.hazmat.backends import default_backend
     from cryptography.hazmat.primitives import hashes
     from cryptography.hazmat.primitives.asymmetric import padding
-    from cryptography.hazmat.primitives.serialization import (
-        load_pem_private_key,
-    )
+    from cryptography.hazmat.primitives.serialization import load_pem_private_key
 
     def _cloud_front_signer_from_pem(key_id, pem):
         if isinstance(pem, str):
@@ -79,7 +88,7 @@ else:
 
 
 @deconstructible
-class S3Boto3StorageFile(File):
+class S3Boto3StorageFile(CompressedFileMixin, File):
     """
     The default file object used by the S3Boto3Storage backend.
 
@@ -133,10 +142,10 @@ class S3Boto3StorageFile(File):
             )
             if 'r' in self._mode:
                 self._is_dirty = False
-                self.obj.download_fileobj(self._file)
+                self.obj.download_fileobj(self._file, Config=self._storage._transfer_config)
                 self._file.seek(0)
             if self._storage.gzip and self.obj.content_encoding == 'gzip':
-                self._file = GzipFile(mode=self._mode, fileobj=self._file, mtime=0.0)
+                self._file = self._decompress_file(mode=self._mode, file=self._file)
         return self._file
 
     def _set_file(self, value):
@@ -231,7 +240,7 @@ class S3Boto3StorageFile(File):
 
 
 @deconstructible
-class S3Boto3Storage(BaseStorage):
+class S3Boto3Storage(CompressStorageMixin, BaseStorage):
     """
     Amazon Simple Storage Service using Boto3
 
@@ -254,12 +263,6 @@ class S3Boto3Storage(BaseStorage):
 
         check_location(self)
 
-        # Backward-compatibility: given the anteriority of the SECURE_URL setting
-        # we fall back to https if specified in order to avoid the construction
-        # of unsecure urls.
-        if self.secure_urls:
-            self.url_protocol = 'https:'
-
         self._bucket = None
         self._connections = threading.local()
 
@@ -272,6 +275,7 @@ class S3Boto3Storage(BaseStorage):
                 signature_version=self.signature_version,
                 proxies=self.proxies,
             )
+        self._transfer_config = TransferConfig(use_threads=self.use_threads)
 
     def get_cloudfront_signer(self, key_id, key):
         return _cloud_front_signer_from_pem(key_id, key)
@@ -313,7 +317,6 @@ class S3Boto3Storage(BaseStorage):
             'custom_domain': setting('AWS_S3_CUSTOM_DOMAIN'),
             'cloudfront_signer': cloudfront_signer,
             'addressing_style': setting('AWS_S3_ADDRESSING_STYLE'),
-            'secure_urls': setting('AWS_S3_SECURE_URLS', True),
             'file_name_charset': setting('AWS_S3_FILE_NAME_CHARSET', 'utf-8'),
             'gzip': setting('AWS_IS_GZIPPED', False),
             'gzip_content_types': setting('GZIP_CONTENT_TYPES', (
@@ -323,7 +326,7 @@ class S3Boto3Storage(BaseStorage):
                 'application/x-javascript',
                 'image/svg+xml',
             )),
-            'url_protocol': setting('AWS_S3_URL_PROTOCOL', 'http:'),
+            'url_protocol': setting('AWS_S3_URL_PROTOCOL', 'https:'),
             'endpoint_url': setting('AWS_S3_ENDPOINT_URL'),
             'proxies': setting('AWS_S3_PROXIES'),
             'region_name': setting('AWS_S3_REGION_NAME'),
@@ -331,6 +334,7 @@ class S3Boto3Storage(BaseStorage):
             'verify': setting('AWS_S3_VERIFY', None),
             'max_memory_size': setting('AWS_S3_MAX_MEMORY_SIZE', 0),
             'default_acl': setting('AWS_DEFAULT_ACL', None),
+            'use_threads': setting('AWS_S3_USE_THREADS', True),
         }
 
     def __getstate__(self):
@@ -403,20 +407,6 @@ class S3Boto3Storage(BaseStorage):
         security_token = self.security_token or lookup_env(self.security_token_names)
         return security_token
 
-    def _clean_name(self, name):
-        """
-        Cleans the name so that Windows style paths work
-        """
-        # Normalize Windows style paths
-        clean_name = posixpath.normpath(name).replace('\\', '/')
-
-        # os.path.normpath() can strip trailing slashes so we implement
-        # a workaround here.
-        if name.endswith('/') and not clean_name.endswith('/'):
-            # Add a trailing slash as it was stripped.
-            clean_name += '/'
-        return clean_name
-
     def _normalize_name(self, name):
         """
         Normalizes the name so that paths like /path/to/ignored/../something.txt
@@ -428,12 +418,8 @@ class S3Boto3Storage(BaseStorage):
         except ValueError:
             raise SuspiciousOperation("Attempted access to '%s' denied." % name)
 
-    def _compress_content(self, content):
-        """Gzip a given string content."""
-        return GzipCompressionWrapper(content)
-
     def _open(self, name, mode='rb'):
-        name = self._normalize_name(self._clean_name(name))
+        name = self._normalize_name(clean_name(name))
         try:
             f = S3Boto3StorageFile(name, mode, self)
         except ClientError as err:
@@ -443,11 +429,11 @@ class S3Boto3Storage(BaseStorage):
         return f
 
     def _save(self, name, content):
-        cleaned_name = self._clean_name(name)
+        cleaned_name = clean_name(name)
         name = self._normalize_name(cleaned_name)
         params = self._get_write_parameters(name, content)
 
-        if not hasattr(content, 'seekable') or content.seekable():
+        if is_seekable(content):
             content.seek(0, os.SEEK_SET)
         if (self.gzip and
                 params['ContentType'] in self.gzip_content_types and
@@ -456,27 +442,35 @@ class S3Boto3Storage(BaseStorage):
             params['ContentEncoding'] = 'gzip'
 
         obj = self.bucket.Object(name)
-        obj.upload_fileobj(content, ExtraArgs=params)
+        obj.upload_fileobj(content, ExtraArgs=params, Config=self._transfer_config)
         return cleaned_name
 
     def delete(self, name):
-        name = self._normalize_name(self._clean_name(name))
-        self.bucket.Object(name).delete()
+        try:
+            name = self._normalize_name(clean_name(name))
+            self.bucket.Object(name).delete()
+        except ClientError as err:
+            if err.response['ResponseMetadata']['HTTPStatusCode'] == 404:
+                # Not an error to delete something that does not exist
+                return
+
+            # Some other error was encountered. Re-raise it
+            raise
 
     def exists(self, name):
-        name = self._normalize_name(self._clean_name(name))
+        name = self._normalize_name(clean_name(name))
         try:
             self.connection.meta.client.head_object(Bucket=self.bucket_name, Key=name)
             return True
-        except ClientError as error:
-            if error.response['ResponseMetadata']['HTTPStatusCode'] == 404:
+        except ClientError as err:
+            if err.response['ResponseMetadata']['HTTPStatusCode'] == 404:
                 return False
 
             # Some other error was encountered. Re-raise it.
             raise
 
     def listdir(self, name):
-        path = self._normalize_name(self._clean_name(name))
+        path = self._normalize_name(clean_name(name))
         # The path needs to end with a slash, but if the root is empty, leave it.
         if path and not path.endswith('/'):
             path += '/'
@@ -495,7 +489,7 @@ class S3Boto3Storage(BaseStorage):
         return directories, files
 
     def size(self, name):
-        name = self._normalize_name(self._clean_name(name))
+        name = self._normalize_name(clean_name(name))
         return self.bucket.Object(name).content_length
 
     def _get_write_parameters(self, name, content=None):
@@ -532,7 +526,7 @@ class S3Boto3Storage(BaseStorage):
         Returns an (aware) datetime object containing the last modified time if
         USE_TZ is True, otherwise returns a naive datetime in the local timezone.
         """
-        name = self._normalize_name(self._clean_name(name))
+        name = self._normalize_name(clean_name(name))
         entry = self.bucket.Object(name)
         if setting('USE_TZ'):
             # boto3 returns TZ aware timestamps
@@ -570,7 +564,7 @@ class S3Boto3Storage(BaseStorage):
 
     def url(self, name, parameters=None, expire=None, http_method=None):
         # Preserve the trailing slash after normalizing the path.
-        name = self._normalize_name(self._clean_name(name))
+        name = self._normalize_name(clean_name(name))
         params = parameters.copy() if parameters else {}
         if expire is None:
             expire = self.querystring_expire
@@ -599,7 +593,7 @@ class S3Boto3Storage(BaseStorage):
 
     def get_available_name(self, name, max_length=None):
         """Overwrite existing file with the same name."""
-        name = self._clean_name(name)
+        name = clean_name(name)
         if self.file_overwrite:
             return get_available_overwrite_name(name, max_length)
         return super().get_available_name(name, max_length)
